A simple web application in Python, using Django
================================================

Part 1
------

Install the virtual environment:

    virtualenv --no-site-packages venv
    source venv/bin/activate.csh
    pip install -r requirements.txt

Create the project:

    django-admin startproject redurl

Edit `redurl/settings.py` to define the database.
Set up things:

    python manage.py migrate

Then start the server:

    python manage.py runserver

Now, create the application:

    python manage.py startapp urlapp

Edit `urlapp/models.py` to define your models.

Add the application in `redurl/settings.py`.
Then create the database migrations corresponding to your models:

    python manage.py makemigrations urlapp

You can see what really happened with:

    python manage.py sqlmigrate urlapp 0001

Finally, run the migrations:

    python manage.py migrate

Show in the interactive shell:

    python manage.py shell
    
    >>> from urlapp.models import User, Url
    >>> User.objects.all()
    []
    >>> u = User(username="nickie", password="sesame")
    >>> u.save()
    >>> u.id
    1
    >>> u.username
    'nickie'
    >>> u.password
    'sesame'
    >>> User.objects.all()
    [<User: nickie>]
    >>> User.objects.get(id=1)
    <User: nickie>
    >>> User.objects.get(pk=1)
    <User: nickie>
    >>> u.url_set.all()
    []
    >>> u.url_set.create(user=u, target="http://www.softlab.ntua.gr/~nickie/")
    <Url: http://www.softlab.ntua.gr/~nickie/>
    >>> u.url_set.all()
    [<Url: http://www.softlab.ntua.gr/~nickie/>]
    >>> u.url_set.count()
    1

Part 2
------

Create admin user:

    python manage.py createsuperuser

Edit `urlapp/admin.py` to allow modifications to the app.

Part 3
------

Remove the `User` model and replace with that in `django.contrib.auth.models`.

    python manage.py makemigrations urlapp
    python manage.py migrate

Part 4
------

Add default view.
Edit `urlapp/views.py`, create `urlapp/urls.py`.
Edit `redurl/urls.py`.

Make views that do something, just reading the objects' data.

Make views using templates.
Edit `urlapp/views.py`.
Create `urlapp/templates/index.html` and `urlapp/templates/details.html`.

Add 404.
Edit `urlapp/views.py`.

Higher-level URLs.
Edit `urlapp/templates/index.html`.
